<?php

/**
 * @file
 * Theming functions for the image_target_question question type.
 */

/**
 * Preprocess the alternative answers to the creation form
 */
function theme_image_target_question_creation_form(&$form) {
  $path = drupal_get_path('module', 'image_target_question');
  jquery_ui_add(array('ui.draggable', 'ui.droppable', 'ui.sortable'));
  _image_target_question_add_js('image_target_question_creation');

  $output = '';

  $output .= drupal_render($form['upload']);
  $output .= drupal_render($form['upload_now']);
  $output .= '<div style="clear: both; padding-bottom: 20px;"></div>';
  $output .= drupal_render($form['image_target_question_image']);
  $output .= drupal_render($form['any_region']);

  if (isset($form['targets'])) {
    $rows = array();
    foreach ($form['targets'] as $identifier => $target) {
      if (intval($identifier) > 0) {
        $row = array();
        $row[] = drupal_render($target['selected_region']);
        $row[] = drupal_render($target['clear_region']) . drupal_render($target['region']);
        $row[] = drupal_render($target['targettext']);
        $row[] = drupal_render($target['score_if_correct']);
        $row[] = drupal_render($target['score_if_not_correct']);
        $rows[] = $row;
      }
    }
    $header[] = array('data' => t('Region'), 'colspan' => 2);
    $header[] = array('data' => t('Description'));
    $header[] = array('data' => t('Score if correct'));
    $header[] = array('data' => t('Score if not correct'));
    $output .= theme('table', $header, $rows, array(), NULL, TRUE);
  }

  $output .= drupal_render($form['feedback']);
  $output .= '<h3>' . t('Instructions') . '</h3>';
  $output .= '<p><strong>' . t('Setting an image') . ':</strong> ' . t('You should first upload an image using the "Attach a new image file" control above. After that you can define the regions on the image that the users must identify..') . '</p>';
  $output .= '<p><strong>' . t('Setting a region') . ':</strong> ' . t('There must be a description for every target you wish to show to the user.  Click "Set region x" then click and drag on your image to set a region on the image. The user gets the "score if correct" score if they put the target inside your region, otherwise they will get the "score if not correct".') . '</p>';
  $output .= '<p><strong>' . t('Setting a non-region') . ':</strong> ' . t('Any region with a description but no region defined is a non-region. The user will be presented with a target to drag and they will get the "correct" score if they do not place it anywhere on the image, otherwise they will get the "score if not correct". If they put the To remove a previously set region click the "Clear region x" button.') . '</p>';
  $output .= '<p><strong>' . t('Any targets to any region') . ':</strong> ' . t('If the user can put any target in any region tick the any target in any region box. If using this option try not have any overlapping regions. The user scores the "score if correct" value for each region containing at least one target.') . '</p>';

  // TODO : Add alternatives functionality.
  return $output;
}

/**
 * Theme the answer part of the node view
 *
 * @param array $alternatives
 *   Array of alternatives. Each alternative is also an array with all the
 *   data for each alternative.
 * @param boolean $show_correct
 *   True if the user is allowed to view the solution
 * @param int $any_region
 *   1 if targets can be placed in any region.
 */
function theme_image_target_question_answer_node_view($alternatives, $show_correct, $fid, $any_region) {
  $header = array('');
  $rows = array();
  $answers = array();

  if ($show_correct) {
    foreach ($alternatives as $i => $short) {
      if ($show_correct) {
        $answers[$i] = array(
          'region' => $short,
        );
      }

      $rows[] = array(
        $i . '. ' . check_plain($short['targettext']),
      );
    }
  }

  $output = theme('image_target_question_image', $fid, $answers, $show_correct);
  $output .= theme('table', $header, $rows);

  return $output;
}

/**
 * Theme function for the image_target_question report.
 * This appears at the end of a quiz and shows the user what they got right
 * and wrong
 *
 * @param int $fid
 *   The file fid of the image_target_question image
 */
function theme_image_target_question_response($fid, array $answers = array(), $feedback = '', $showcorrect = FALSE, $showfeedback = FALSE, $any_region = FALSE) {
  $rows = array();
  $path = drupal_get_path('module', 'image_target_question');
  $img_path = $path . '/theme/images';
  drupal_add_css($path . '/theme/image_target_question.css');
  $all_correct = TRUE;

  foreach ($answers as $identifier => $answer) {
    $cols = array();
  
    if (isset($answer['region'])) {
      $cols[] = array(
        'data' => "<span>{$identifier}. </span>",
        'width' => 35,
      );

      if ($showcorrect) {
        $info = $answer['correct']
          ? theme('image', "{$img_path}/correct.png", t('Correct'), t('This answer is correct'), array('class' => 'feedback-icon'))
          : theme('image', "{$img_path}/wrong.png", t('Wrong'), t('This answer is wrong'), array('class' => 'feedback-icon'));

        $cols[] = array(
          'data' => $info,
          'width' => 35,
          'rowspan' => 1,
          'class' => 'selector-td',
        );
      }

      $cols[] = array(
        'data' => $answer['targettext'],
      );

    $rows[] = $cols;
    }
    else {
      $all_correct = $answer['correct'] ? $all_correct : FALSE;
    }
  }

  if (empty($rows)) {
    if ($showcorrect) {
      $info = $all_correct
          ? theme('image', "{$img_path}/correct.png", t('Correct'), t('This answer is correct'), array('class' => 'feedback-icon'))
          : theme('image', "{$img_path}/wrong.png", t('Wrong'), t('This answer is wrong'), array('class' => 'feedback-icon'));

      $cols[] = '';
      $cols[] = array(
        'data' => $info,
        'width' => 35,
        'rowspan' => 1,
        'class' => 'selector-td',
      );
      $cols[] = array(
        'data' => t('The image did not have anything to identify in it.'),
      );
      $rows[] = $cols;
    }
    else {
      $rows[] = array(array('colspan' => 3, 'data' => '<strong>' . t('The image did not have anything to identify in it.') . '</strong>'));
    }
  }

  if ($showfeedback) {
    $rows[] = array(array('colspan' => 3, 'data' => '<strong>' . t('Feedback:') . '</strong><br />' . $feedback));
  }

  $table = theme('table', NULL, $rows);
  $img = is_numeric($fid) ? theme('image_target_question_image', $fid, $answers, $showcorrect, $showfeedback, $any_region) : '';
  return $img . $table;
}

/**
 * Theme the img tag for the image_target_question image
 *
 * @param int $fid
 *   The fid of the actual uploaded image. If imagecache is setup, the img
 *   returned will be the image cached one
 * @param array $answers
 *   One for each answer with the following keys
 *   'region' - An answer region on the image:
 *   ARRAY [INT 'leftpx', INT 'width', INT 'top', INT 'height']
 *   'target' (optional) - Where to place the target marker:
 *   ARRAY [INT 'top', INT 'left'] relative to the top left of the image
 * @param boolean $showpoints
 *   TRUE to display the answers (regions)
 * @param boolean $showfeedback
 *   TRUE to display the feedback (note this is not implemented, only relevant
 *   when there is separate feedback per answer)
 *
 * @return string
 *   HTML displaying the image_target_question image and (optionally) answer
 *   regions and user targets
 */
function theme_image_target_question_image($fid, array $answers = array(), $showpoints = FALSE, $showfeedback = FALSE, $any_region = FALSE) {
  if (is_numeric($fid) && $fid > 0) {
    $file = image_target_question_file_load($fid);
    $url = image_target_question_get_imagecache_preset() ? imagecache_create_url(image_target_question_get_imagecache_preset()->presetname, $file->filepath) : url($file->filepath);

    $output = '<div id="image_target_question_image" style="position: relative; height: ' . image_target_question_get_image_height($fid) . 'px;">';
    $output .= '<img src="' . $url . '" style="position: absolute;" />';

    foreach ($answers as $identifier => $answer) {
      // Region.
      if ($showpoints && $region['width'] > 0 && $region['height'] > 0) {
        $region = $answer['region'];
        $output .= theme('image_target_question_region', $region['top'], $region['leftpx'], $region['width'], $region['height'], $identifier);
      }

      // Target.
      if (isset($answer['target']) && ($answer['target']['top'] > 0 && $answer['target']['leftpx'] > 0)) {
        $target = $answer['target'];
        $output .= theme('image_target_question_target', $target['top'], $target['leftpx'], $any_region ? '' : $identifier);
      }
    }
    $output .= '</div>';
    return $output;
  }
  return '';
}

/**
 * Theme template preprocess for the targets that get dragged onto the form
 * Override this function in your template if you want to change the targets
 * Be sure to offset the top and left co-ordinates by half the width or height
 * of the target image so it gets centered on the area the drag onto
 * e.g. The default target used by the module is 32px x 32px so top and left
 * co-ordinates are reduced by 16.
 */
function template_preprocess_image_target_question_target(&$vars) {
  $path = drupal_get_path('module', 'image_target_question') . '/theme/images';
  $vars['top'] = $vars['top'] - 16;
  $vars['left'] = $vars['left'] - 16;
  $vars['width'] = 32;
  $vars['height'] = 32;
  $vars['imgurl'] = url($path . '/target.png');
}

/**
 * Theme function - describe output of the answering page for the
 * image_target_question
 */
function theme_image_target_question_answering_form($form) {
  $path = drupal_get_path('module', 'image_target_question');
  jquery_ui_add(array('ui.draggable', 'ui.droppable', 'ui.sortable'));
  _image_target_question_add_js('image_target_question_answer');
  drupal_add_css($path . '/theme/image_target_question.css');

  $header = array(
    array('data' => t('Target Number')),
    array('data' => t('Drag Target')),
    array('data' => t('Description')),
  );
  $rows = array();

  foreach ($form['targets'] as $identifier => $target) {
    if (intval($identifier) > 0) {
      $target_div = '<div style="position:relative;"><div class="image_target_question-target" id="image_target_question-target-' . $identifier . '" style="position: absolute; width: 32px; height: 32px; top: -14px; background: url(' . url($path . '/theme/images/target.png') . ') no-repeat scroll 0 0 transparent;">' . ($form['any_region']['#value'] ? '' : $identifier) . '</div></div>';
      $row = array();
      $row[] = array(
        'data' => "<span>{$identifier}. </span>",
        'width' => 35,
      );
      $row[] = array(
        'class' => 'drag-' . $identifier,
        'data' => $target_div,
        'width' => 35,
      );
      $row[] = drupal_render($target['targettext']);
      $rows[] = $row;
    }
  }
  $output = drupal_render($form['question']);
  $output .= drupal_render($form['image_target_question_image']);
  $output .= drupal_render($form['tries[answer]']);

  $output .= "<ul class='instructions'>";
  $output .= '<li>' . t('Drag the cross hair targets below onto the image to identify the regions shown.') . '</li>';
  $output .= '<li>' . ($any_region ? t('You can place any target in any region to identify them.') : t('Make sure you identify the correct cross hair target with the correct region of the image')) . '</li>';
  $output .= '<li>' . t('Not all the regions may be on the image so make sure you only place the cross hair if you are sure.') . '</li>';
  $output .= '</ul>';

  $output .= theme('table', $header, $rows, array(), NULL, TRUE);
  $output .= drupal_render($form['navigation']);

  return $output;
}
